@@ -822,6 +822,9 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
 
                 stages_rebal_df['risky_size_mark_to_market'] = compute_mark_to_market_risky_assets(to_keep_init_eth,to_keep_init_liq,to_keep_init_price,data_df=stages_rebal_df.copy())
 
+                stages_rebal_df['new_btcs'].iloc[0]=to_keep_init_btc
+                stages_rebal_df['new_ethers'].iloc[0]=to_keep_init_eth
+
                 stages_rebal_df['new_btcs'] = stages_rebal_df['new_btcs'].ffill()
                 stages_rebal_df['new_ethers'] = stages_rebal_df['new_ethers'].ffill()
                 stages_rebal_df['new_liquidities'] = stages_rebal_df['new_liquidities'].ffill()
@@ -832,6 +835,8 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
                 stages_rebal_df['new_liquidities'] = stages_rebal_df['new_liquidities'].bfill()
                 stages_rebal_df['swapping_sizes'] = stages_rebal_df['swapping_sizes'].bfill()
 
+
+
                 stages_rebal_df = stages_rebal_df.rename(columns = {'new_btcs':'numeraire_size',
                                                  'new_ethers':'risky_size',
                                                  'new_liquidities':'liquidity_size'})
@@ -862,7 +867,7 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
                 #all_df[['incurred_v3', 'lp_position_profit', 'we_rebalance_signal', 'preceding_vault_signal','flipped_lp_position_profit']]
 
                 print('computing the gain and losses from all in liguidity')
-                all_df['mark_to_market_loss_v3'] = 100. * (1.+all_df['curILv3'].values)
+
                 all_df['mark_to_market_loss_v2'] = 100. * (1.+all_df['curILv2'].values)
 
 
@@ -870,6 +875,29 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
                 all_df['loss_v3'] = 100. * np.cumprod(1 + all_df['incurred_v3'].values)
                 all_df['loss_v2'] = 100. * np.cumprod(1 + all_df['incurred_v2'].values)
 
+                all_df['curILv3'] = all_df['curILv3'].fillna(0.)
+                all_df['rebalancing'] = all_df['rebalancing'].fillna(False)
+
+                def compute_loss_v3_mark_to_market(data_df = None):
+                    previous_valo = 100.
+                    valos = np.zeros(len(data_df))
+                    counter = 0
+                    for i, row in data_df.iterrows():
+                        rebal = row['rebalancing']
+                        current_valo_since_rebal = row['curILv3']
+                        valos[counter]= previous_valo * (1. + current_valo_since_rebal)
+                        if rebal :
+                            previous_valo = valos[counter]
+                        counter = counter + 1
+                    return valos
+
+                all_df['mark_to_market_loss_v3'] = compute_loss_v3_mark_to_market(data_df=all_df[['incurred_v3', 'curILv3', 'rebalancing']].copy())
+                all_df['mark_to_market_loss_v3_returns'] = all_df['mark_to_market_loss_v3'].pct_change()
+                all_df['mark_to_market_loss_v3_returns'] = all_df['mark_to_market_loss_v3_returns'].fillna(0.)
+
+                all_df['mark_to_market_loss_v3_without_reset'] = 100. * (1. + all_df['curILv3'].values)
+
+
                 all_df['cumulated_apr'] =  100. * np.cumprod(1 + all_df['apr'].values)
                 all_df['cumulated_swapping_costs'] =  100. * np.cumprod(1  - all_df['swapping_cost'].values)
 
@@ -904,17 +932,48 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
                 all_df['mark_to_market_lp_value_returns'] = all_df['mark_to_market_lp_value'].pct_change()
                 all_df['mark_to_market_lp_value_returns'] = all_df['mark_to_market_lp_value_returns'].fillna(0.)
 
-                all_df['gain_and_value_LP_LS'] = 100. * np.cumprod(
+                all_df['performance_LP_LS'] = 100. * np.cumprod(
                     1 + all_df['flipped_lp_position_profit'].values + all_df['apr'].values - all_df['swapping_cost'].values)
-                all_df['gain_and_value_LP_LO'] = 100. * np.cumprod(
+                all_df['performance_LP_LO'] = 100. * np.cumprod(
                     1 + all_df['lp_position_profit'].values + all_df['apr'].values - all_df['swapping_cost'].values)
 
-                all_df['gain_and_value_LP_LO_mark_to_market'] = 100. * np.cumprod(1 + all_df['mark_to_market_lp_value_returns'].values + all_df['apr'].values - all_df['swapping_cost'].values)
+                all_df['performance_LP_LO_mark_to_market'] = 100. * np.cumprod(1 + all_df['mark_to_market_lp_value_returns'].values + all_df['apr'].values - all_df['swapping_cost'].values)
+
+                all_df['performance_LP_LO'] = 100. * np.cumprod(
+                    1 + all_df['lp_position_profit'].values + all_df['apr'].values - all_df['swapping_cost'].values)
+
+                all_df['performance_ILV3_mark_to_market'] = 100. * np.cumprod(1 + all_df['mark_to_market_loss_v3_returns'].values + all_df['apr'].values - all_df['swapping_cost'].values)
+                all_df['performance_ILV3'] = 100. * np.cumprod(1 + all_df['incurred_v3'].values + all_df['apr'].values - all_df['swapping_cost'].values)
 
                 #all_df['gain_and_loss_v3'] = 100. * np.cumprod(1 + all_df['incurred_v3'].values + all_df['apr'].values)
                 #all_df['gain_and_loss_v2'] = 100. * np.cumprod(1 + all_df['incurred_v2'].values + all_df['apr'].values )
 
+                all_df['performance_against_risky_mark_to_market'] = all_df['performance_LP_LO_mark_to_market']/all_df['Close']
+                all_df['performance_against_risky'] = all_df['performance_LP_LO']/all_df['Close']
+
+
                 if plotHtml:
+                    fig = realtime_plotting_utility.plot_multiple_time_series(
+                        data_df=all_df[['performance_against_risky_mark_to_market','performance_against_risky']].copy(), logy=False, split=False,
+                        put_on_same_scale=False,
+                        title=f'Performance against a full risky HODL basket {ssj} {ssj_against}')
+                    fig.show()
+                    fig = realtime_plotting_utility.plot_multiple_time_series(
+                        data_df=all_df[['performance_ILV3_mark_to_market']].copy(), logy=False, split=False,
+                        put_on_same_scale=False,
+                        title=f'Performance against a 50/50 HODL basket {ssj} {ssj_against}')
+                    fig.show()
+
+                    fig = realtime_plotting_utility.plot_multiple_time_series(
+                        data_df=all_df[['performance_ILV3_mark_to_market','performance_ILV3']].copy(), logy=False, split=False,
+                        put_on_same_scale=False,
+                        title=f'Performance against a 50/50 HODL basket {ssj} {ssj_against}')
+                    fig.show()
+                    fig = realtime_plotting_utility.plot_multiple_time_series(
+                        data_df=all_df[['performance_ILV3_mark_to_market']].copy(), logy=False, split=False,
+                        put_on_same_scale=False,
+                        title=f'Performance against a 50/50 HODL basket {ssj} {ssj_against}')
+                    fig.show()
                     fig = realtime_plotting_utility.plot_multiple_time_series(
                         data_df=all_df[['cumulated_swapping_costs']].copy(), logy=False, split=False,
                         put_on_same_scale=False,
@@ -954,7 +1013,7 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
                     fig = realtime_plotting_utility.plot_multiple_time_series(
                         data_df=all_df[['loss_v3', 'mark_to_market_loss_v3']].copy(), logy=False, split=False,
                         put_on_same_scale=False,
-                        title=f'loss V3 {ssj} {ssj_against}')
+                        title=f'impermanent loss V3 {ssj} {ssj_against}')
                     fig.show()
 
                     fig = realtime_plotting_utility.plot_multiple_time_series(
@@ -1005,7 +1064,7 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
 
                     fig1 = realtime_plotting_utility.plot_multiple_time_series(
                         #                    data_df=all_df[['loss_v2', 'loss_v3', 'gain_and_loss_v3','gain_and_loss_LS']].copy(),
-                        data_df=all_df[['gain_and_value_LP_LS', 'gain_and_value_LP_LO']].copy(),
+                        data_df=all_df[['performance_LP_LS', 'performance_LP_LO']].copy(),
                         logy=False, split=False,
                         put_on_same_scale=False,
                         title=f'Absolute performance LO/LS {ssj} {ssj_against}')
@@ -1013,7 +1072,7 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
 
                     fig1 = realtime_plotting_utility.plot_multiple_time_series(
                         #                    data_df=all_df[['loss_v2', 'loss_v3', 'gain_and_loss_v3','gain_and_loss_LS']].copy(),
-                        data_df=all_df[['gain_and_value_LP_LO_mark_to_market', 'gain_and_value_LP_LO']].copy(),
+                        data_df=all_df[['performance_LP_LO_mark_to_market', 'performance_LP_LO']].copy(),
                         logy=False, split=False,
                         put_on_same_scale=False,
                         title=f'Absolute performance v3 liquidity providing {ssj} {ssj_against}')
@@ -1021,7 +1080,7 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
 
                     fig1 = realtime_plotting_utility.plot_multiple_time_series(
                         #                    data_df=all_df[['loss_v2', 'loss_v3', 'gain_and_loss_v3','gain_and_loss_LS']].copy(),
-                        data_df=all_df[['gain_and_value_LP_LO']].copy(),
+                        data_df=all_df[['performance_LP_LO']].copy(),
                         logy=False, split=False,
                         put_on_same_scale=False,
                         title=f'Absolute performance v3 liquidity providing {ssj} {ssj_against}')
@@ -1054,7 +1113,7 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
                 on_mode = (all_df['providingLiquidity'] == 1).sum()/len(all_df)
                 from cryptotoolbox.risk_metrics import riskmetrics
 
-                kpi_df = riskmetrics.get_kpi(all_df[[f'gain_and_value_LP_LO_mark_to_market']])
+                kpi_df = riskmetrics.get_kpi(all_df[[f'performance_LP_LO_mark_to_market']])
                 obf_data= {
                     'bb_period': bb_period,
                     'stdNbr': stdNbr,
@@ -1065,7 +1124,7 @@ def LS_IL_boundaries_with_signal(data_df = None, plotHtml = False, me_pair = ('E
                     'v2_loss_apy': v2_loss_apy,
                     'v3_loss_apy': v3_loss_apy,
                 }
-                obf_data.update(kpi_df.to_dict()['gain_and_value_LP_LO_mark_to_market'])
+                obf_data.update(kpi_df.to_dict()['performance_LP_LO_mark_to_market'])
 
                 objectives.append(obf_data)
 
